#include "acpi/RSDT.h"
#include "bootboot.h"
#include "std/stdio.h"
#include "std/string.h"

extern BOOTBOOT bootboot;

ACPI::SDTHeader* ACPI::GetRSDTOrXSDT()
{
    return (SDTHeader*)bootboot.arch.x86_64.acpi_ptr;
}

bool ACPI::ValidateSDTHeader(ACPI::SDTHeader* header)
{
    uint8_t sum = 0;

    for (uint32_t i = 0; i < header->Length; i++) { sum += ((char*)header)[i]; }

    return sum == 0;
}

void* ACPI::FindTable(ACPI::SDTHeader* rootSDT, const char* signature)
{
    bool isXSDT = strncmp(rootSDT->Signature, "XSDT", 4);
    int entries = (rootSDT->Length - sizeof(SDTHeader)) / isXSDT ? 8 : 4;

    printf("%d entries in root SDT\n", entries);

    for (int i = 0; i < entries; i++)
    {
        printf("Trying this entry\n");
        SDTHeader* h;
        if (isXSDT) h = (SDTHeader*)((XSDT*)rootSDT)->PointerToOtherSDT[i];
        else
            h = (SDTHeader*)(uint64_t)((RSDT*)rootSDT)->PointerToOtherSDT[i];
        if (strncmp(h->Signature, signature, 4) == 0) return (void*)h;
    }

    return NULL;
}